-- regex_flags[]
regex_flags = ifre.flags()


-- regex_cache[]
regex_cache = {}


-- regex_cache[] metatable
regex_cache.mt = {}
setmetatable(regex_cache, regex_cache.mt)


-- regex_cache[] metamethods
regex_cache.mt.__index = function (cache, key)
    local _, _, pattern, cflags = string.find(key, '^(.*)%z(.*)$')

    local _, compiled = ifre.compile(pattern, tonumber(cflags))
    
    cache[key] = compiled

    return compiled
end

regex_cache.mt.__gc = function (element)
    ifre.free(element)
end


-- regex_search()
function regex_search(pattern, subject, cflags, eflags)
    check_type(pattern, 'string')
    check_type(subject, 'string')

    if (cflags ~= nil) then
	check_type(cflags, 'table')
    else
	cflags = {}
    end

    if (eflags ~= nil) then
	check_type(eflags, 'table')
    else
	eflags = {}
    end

    cf = 0
    for _, f in ipairs(cflags) do
	cf = cf + regex_flags[f]
    end

    ef = 0
    for _, f in pairs(eflags) do
	ef = cf + regex_flags[f]
    end

    compiled = regex_cache[pattern .. '\0' .. cf]

    if (compiled == nil) then
	return nil
    end
    
    return ifre.exec(compiled, subject, ef)
end
